package cat.hack3.codingtests.marsrover;

import cat.hack3.codingtests.marsrover.cartography.Coordinates;
import cat.hack3.codingtests.marsrover.cartography.Direction;
import cat.hack3.codingtests.marsrover.cartography.MarsMap;

public class MarsRover implements RotableRiderRover{

    private final MarsMap marsMap;
    private final RoverActionReporter roverActionReporter;
    private Direction currentDirection;

    public MarsRover(MarsMap marsMap, Direction startingDirection) {
        this.marsMap = marsMap;
        currentDirection = startingDirection;
        roverActionReporter = new RoverActionReporter();
    }

    @Override
    public void moveTowards(Direction direction) {
        Coordinates newPosition = marsMap.getNextPositionTowards(direction);
        boolean willCollide = marsMap.willCollideWithObstacle(newPosition);
        if (willCollide) {
            roverActionReporter.reportObstacle(newPosition);
        } else {
            moveTo(direction);
            marsMap.updateLocation(newPosition);
            roverActionReporter.reportMovement(direction, newPosition);
        }
    }

    private void moveTo(Direction direction) {
        //physical move (not implemented)
    }

    @Override
    public void rotateTowards(Rotation rotation) {
        var newDirection = switch (rotation) {
            case LEFT -> currentDirection.getNextDirectionRotatingMinus90Degrees();
            case RIGHT -> currentDirection.getNextDirectionRotating90Degrees();
        };
        rotateTo(rotation);
        updateDirection(newDirection);
        roverActionReporter.reportRotation(rotation, newDirection);
    }

    private void rotateTo(Rotation rotation) {
        //physical rotation (not implemented)
    }

    private void updateDirection(Direction newDirection) {
        currentDirection = newDirection;
    }

    @Override
    public Coordinates getCurrentCoordinates() {
        return marsMap.getCurrentPosition();
    }

    @Override
    public Direction getCurrentDirection() {
        return currentDirection;
    }
}
