package cat.hack3.codingtests.marsrover.ui.console;

import cat.hack3.codingtests.marsrover.*;

import java.util.Scanner;

import static cat.hack3.codingtests.marsrover.ui.console.UICommons.isNotExitSignal;
import static cat.hack3.codingtests.marsrover.ui.console.UICommons.output;

public class RoverCommandsPerformer {
    private final Scanner reader;
    private final RoverCommand moveForwardCommand;
    private final RoverCommand moveBackwardsCommand;
    private final RoverCommand turnLeftCommand;
    private final RoverCommand turnRightCommand;

    public RoverCommandsPerformer(Scanner reader, RotableRiderRover rover) {
        this.reader = reader;
        moveForwardCommand = new MoveForwardCommand(rover);
        moveBackwardsCommand = new MoveBackwardsCommand(rover);
        turnLeftCommand = new TurnLeftCommand(rover);
        turnRightCommand = new TurnRightCommand(rover);
    }

    public void acceptCommandsUntilExitSignal() {
        var nextCommand = "none";
        do {
            output("Insert command (f = forward, b = backward, l = turn left, r = turn right, q = quit):");
            nextCommand = reader.next();
            switch (nextCommand) {
                case "f" -> moveForwardCommand.execute();
                case "b" -> moveBackwardsCommand.execute();
                case "l" -> turnLeftCommand.execute();
                case "r" -> turnRightCommand.execute();
                case "q" -> output("Disconnecting from Mars...");
                default -> output("unknown command");
            }
        } while (isNotExitSignal(nextCommand));
    }
}
